from swarmauri.parsers.concrete.BeautifulSoupElementParser import (
    BeautifulSoupElementParser,
)
from swarmauri.parsers.concrete.BERTEmbeddingParser import BERTEmbeddingParser
from swarmauri.parsers.concrete.CSVParser import CSVParser
from swarmauri.parsers.concrete.EntityRecognitionParser import EntityRecognitionParser
from swarmauri.parsers.concrete.HTMLTagStripParser import HTMLTagStripParser
from swarmauri.parsers.concrete.KeywordExtractorParser import KeywordExtractorParser
from swarmauri.parsers.concrete.Md2HtmlParser import Md2HtmlParser
from swarmauri.parsers.concrete.OpenAPISpecParser import OpenAPISpecParser
from swarmauri.parsers.concrete.PhoneNumberExtractorParser import (
    PhoneNumberExtractorParser,
)
from swarmauri.parsers.concrete.PythonParser import PythonParser
from swarmauri.parsers.concrete.RegExParser import RegExParser
from swarmauri.parsers.concrete.TextBlobNounParser import TextBlobNounParser
from swarmauri.parsers.concrete.TextBlobSentenceParser import TextBlobSentenceParser
from swarmauri.parsers.concrete.URLExtractorParser import URLExtractorParser
from swarmauri.parsers.concrete.XMLParser import XMLParser
